#include "player.h"
#include "hero.h"
#include "map1.h"
#include "projectile.h"

/****************************************INICIALIZAR PERSONAJE****************************************/

//Definicion
TPlayer player;

//Array de sprites
#define PLAYER_FRAMES 15

const TSprite g_frames[PLAYER_FRAMES] = {
	{ RIGHT, g_hero_00 } , { RIGHT, g_hero_01 } ,
    { RIGHT, g_hero_02 } , { RIGHT, g_hero_03 } ,
    { RIGHT, g_hero_04 } , { RIGHT, g_hero_05 } ,
    { RIGHT, g_hero_06 } , { RIGHT, g_hero_07 } ,
    { RIGHT, g_hero_08 } , { RIGHT, g_hero_09 } ,
    { RIGHT, g_hero_10 } , { RIGHT, g_hero_11 } ,
    { RIGHT, g_hero_12 } , { RIGHT, g_hero_13 } ,
    { RIGHT, g_hero_14 }
};

//Animaciones
#define PAUSE 4
#define FRAMES_MOVING 8
#define FRAMES_JUMPING 1
#define FRAMES_DOUBLEJUMP 4

TSprite* const anim_moving[FRAMES_MOVING] = {
	&g_frames[0], &g_frames[1], &g_frames[2], &g_frames[3], 
	&g_frames[4], &g_frames[5], &g_frames[6], &g_frames[7]
};

TSprite* const anim_djumping[FRAMES_DOUBLEJUMP] = {
	&g_frames[10], &g_frames[11], &g_frames[12], &g_frames[13]
};

TSprite* const anim_jumping = &g_frames[14];

//Inicializar las variables del personaje
void player_initialize() {
	player.x = player.px = player.ppx = 20;
	player.y = player.py = player.ppy = 157;
	player.y = (player.y & 0b11111100) + 2;
	player.vy = 0;
	player.facing = RIGHT;
	player.sprite = &g_frames[0];
	player.state = ST_STILL;
	player.nframe = 0;
	player.moving  = NO;
	player.pShoot = NO;
	player.djumping = NO;
	player_draw();
}

/****************************************DIBUJAR Y BORRAR SPRITE****************************************/

//Tabla de transparencias
cpctm_declareMaskTable(g_transTable);

extern u8 *g_backBuffer;

//Dibujar el sprite
void player_draw() {
	u8* pvmem = cpct_getScreenPtr(g_backBuffer, player.x, player.y);
	cpct_drawSpriteMaskedAlignedTable (player.sprite->img, pvmem, G_HERO_00_W, G_HERO_00_H, g_transTable);
}

//Borrar el sprite
void player_delete() {
	u8* pvmem = cpct_getScreenPtr(g_backBuffer, 0, ORIGEN_MAPA_Y);
	u8 w = 5 + (player.ppx & 1);
	u8 h = 6 + (player.ppy & 3 ? 1 : 0);
	cpct_etm_drawTileBox2x4 (player.ppx / 2, (player.ppy - ORIGEN_MAPA_Y)/4, w, h, 40, pvmem, mapa);
}

void player_drawUpdate(){
	player.ppx = player.px;
	player.ppy = player.py;
	player.px = player.x;
	player.py = player.y;
	--player.moving;
}

//Redibujar el sprite
void player_redraw() {
	player_delete();
	player_draw();
	player_drawUpdate();
}

/****************************************ESTADOS Y ANIMACIONES****************************************/

void player_checkState(){
	switch(player.state) {
      case ST_STILL: player_still(); break;
      case ST_MOVING: player_moving(); break;
      case ST_JUMPING: player_jumping(); break;
      case ST_FALLING: player_falling(); break;
      case ST_DJUMPING: player_djumping(); break;
      case ST_DFALLING: player_dfalling(); break;
   }
}

void player_still(){
	static const cpct_keyID keys[3] = {Key_CursorUp, Key_CursorRight, Key_CursorLeft};
	u8 k = player_checkKeys(keys, 3);
	switch(k) {
	    case 0: break;
	    case 1: player_newState(ST_JUMPING); break;
	    case 2: player_newState(ST_MOVING); break;
	    case 3: player_newState(ST_MOVING); break;
   }
   	if(cpct_isKeyPressed(Key_Space))
        player_shoot();
}

void player_moving(){
	static const cpct_keyID keys[3] = {Key_CursorUp, Key_CursorRight, Key_CursorLeft};
	u8 k = player_checkKeys(keys, 3);
	switch(k) {
	    case 0: player_newState(ST_STILL); break;
	    case 1: player_newState(ST_JUMPING); break;
	    case 2: player_moveRight(); player_animateMoving(RIGHT); break;
	    case 3: player_moveLeft(); player_animateMoving(LEFT); break;
	}
	if(cpct_isKeyPressed(Key_Space))
        player_shoot();

	if(!player_onGround())
		player_newState(ST_FALLING);
}

void player_jumping(){
	static const cpct_keyID keys[2] = {Key_CursorRight, Key_CursorLeft};
	u8 k = player_checkKeys(keys, 2);
	switch(k) {
    	case 0: break;
    	case 1: player_moveRight(); break;
    	case 2: player_moveLeft(); break;
  	}
  	if(cpct_isKeyPressed(Key_Space))
        player_shoot();

  	player_jump();
  	player_newState(ST_FALLING);
}

void player_djumping(){
	static const cpct_keyID keys[2] = {Key_CursorRight, Key_CursorLeft};
	u8 k = player_checkKeys(keys, 2);
	switch(k) {
    	case 0: break;
    	case 1: player_moveRight(); break;
    	case 2: player_moveLeft(); break;
  	}
  	player_jump();
  	player_newState(ST_DFALLING);
  	//player.djumping = YES;
}

void player_falling(){
	static const cpct_keyID keys[2] = {Key_CursorRight, Key_CursorLeft};
	u8 k = player_checkKeys(keys, 2);
	switch(k) {
	    case 0: break;
	    case 1: player_moveRight(); break;
	    case 2: player_moveLeft(); break;
	}
	if(cpct_isKeyPressed(Key_Space))
        player_shoot();

	if(!player.djumping)
		player_checkDJumping();
	else{
		if(cpct_isKeyPressed(Key_CursorUp)){
			player_newState(ST_DJUMPING);
			player.nframe = 0;
		}
	}

	player_fall();

	if(player_onGround()){
		player_setOnGround();
		player_newState(ST_STILL);
		player.djumping = NO;
	}
	player.moving = 2;
}

void player_dfalling(){
	static const cpct_keyID keys[2] = {Key_CursorRight, Key_CursorLeft};
	u8 k = player_checkKeys(keys, 2);
	switch(k) {
	    case 0: break;
	    case 1: player_moveRight(); break;
	    case 2: player_moveLeft(); break;
	}
	player_fall();
	player_animateDFalling(/*player.facing*/);
	if(player_onGround()){
		player_setOnGround();
		player_newState(ST_STILL);
		player.djumping = NO;
	}
	player.moving = 2;
}

void player_newState(u8 state){
	player.state = state;
	player.moving = 2;
}

u8 player_checkKeys(const cpct_keyID* k, u8 numk) {
	u8 i;
	cpct_scanKeyboard();
	if (cpct_isAnyKeyPressed()) {
		for(i=1; i <= numk; i++, k++) {
			if (cpct_isKeyPressed(*k))
	        	return i;
		}
	}
	return 0;
}

void player_checkDJumping() {
	//cpct_scanKeyboard();
	if (cpct_isKeyPressed(Key_CursorUp))
		player.djumping = NO;
	else
		player.djumping = YES;
}

void player_setFrame(TSprite **anim) {
   player.sprite = anim[player.nframe / PAUSE];
}

void player_animateMoving(i8 facing) {
	player.facing = facing;
	if(++player.nframe == FRAMES_MOVING * PAUSE)
		player.nframe = 0;
}

void player_animateDFalling(/*i8 facing*/) {
	//player.facing = facing;
	if(++player.nframe == FRAMES_DOUBLEJUMP * PAUSE)
		player.nframe = 0;
}

void player_selectSprite(){
	TSprite *s;
	switch(player.state){
		case ST_STILL: player.sprite = &g_frames[0]; break;
		case ST_MOVING: player_setFrame(anim_moving); break;
		case ST_JUMPING: player.sprite = &g_frames[14]; break;
		case ST_FALLING: player.sprite = &g_frames[14]; break;
		case ST_DJUMPING: player.sprite = &g_frames[10]; break;
		case ST_DFALLING: player_setFrame(anim_djumping); break;
	}
	s = player.sprite;
	if (s->dir != player.facing){
    	cpct_hflipSpriteM0(G_HERO_00_W, G_HERO_00_H, s->img);
	    s->dir = player.facing;
	}
}

u8* obtenerTilePtr(u8 x, u8 y) {
   return mapa + (y-ORIGEN_MAPA_Y)/4*g_map1_W + x/2;
}

void player_setOnGround() {
   player.y = (player.y & 0b11111100) + 2;
}

u8 player_onGround() {
	u8* tileSuelo = obtenerTilePtr(player.x, player.y + G_HERO_00_H+1);
	if ((*tileSuelo != 11 && *tileSuelo != 1) ||
		(*(tileSuelo+G_HERO_00_W/4) != 11) ||
		(*(tileSuelo+G_HERO_00_W/2) != 11))
		return 1;
	return 0;
}

u8 player_onCeiling() {
	u8* tileTecho = obtenerTilePtr(player.x, player.y-1);
	if (*tileTecho == 2 || *(tileTecho+G_HERO_00_W/2) == 2 ||
		*tileTecho == 3 || *(tileTecho+G_HERO_00_W/2) == 3 ||
		*tileTecho == 6 || *(tileTecho+G_HERO_00_W/2) == 6 ||
		*tileTecho == 7 || *(tileTecho+G_HERO_00_W/2) == 7 ||
		*tileTecho == 1 || *(tileTecho+G_HERO_00_W/2) == 1 ||
		*tileTecho == 5 || *(tileTecho+G_HERO_00_W/2) == 5 ||
		*tileTecho == 8 || *(tileTecho+G_HERO_00_W/2) == 8 ||
		*tileTecho == 9 || *(tileTecho+G_HERO_00_W/2) == 9 ||
		*tileTecho == 12 || *(tileTecho+G_HERO_00_W/2) == 12 ||
		*tileTecho == 13 || *(tileTecho+G_HERO_00_W/2) == 13 )
		return 1;
	return 0;
}

u8 player_onWallLeft() {
	u8* tilePared = obtenerTilePtr(player.x-1, player.y);
	if (*tilePared == 1 || *(tilePared+(G_HERO_00_H-18)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 1 ||
		*tilePared == 5 || *(tilePared+(G_HERO_00_H-18)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 5 ||
		*tilePared == 3 || *(tilePared+(G_HERO_00_H-18)/4*40) == 3 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 3 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 3 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 3 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 3 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 3 ||
		*tilePared == 7 || *(tilePared+(G_HERO_00_H-18)/4*40) == 7 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 7 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 7 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 7 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 7 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 7 ||
		*tilePared == 9 || *(tilePared+(G_HERO_00_H-18)/4*40) == 9 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 9 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 9 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 9 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 9 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 9 ||
		*tilePared == 13 || *(tilePared+(G_HERO_00_H-18)/4*40) == 13 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 13 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 13 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 13 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 13 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 13)
		return 1;
	return 0;
}

u8 player_onWallRight() {
	u8* tilePared = obtenerTilePtr(player.x+G_HERO_00_W, player.y);
	if (*tilePared == 1 || *(tilePared+(G_HERO_00_H-18)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 1 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 1 ||
		*tilePared == 5 || *(tilePared+(G_HERO_00_H-18)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 5 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 5 ||
		*tilePared == 2 || *(tilePared+(G_HERO_00_H-18)/4*40) == 2 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 2 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 2 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 2 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 2 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 2 ||
		*tilePared == 6 || *(tilePared+(G_HERO_00_H-18)/4*40) == 6 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 6 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 6 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 6 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 6 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 6 ||
		*tilePared == 8 || *(tilePared+(G_HERO_00_H-18)/4*40) == 8 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 8 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 8 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 8 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 8 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 8 ||
		*tilePared == 12 || *(tilePared+(G_HERO_00_H-18)/4*40) == 12 ||
						   *(tilePared+(G_HERO_00_H-14)/4*40) == 12 ||
						   *(tilePared+(G_HERO_00_H-10)/4*40) == 12 ||
						   *(tilePared+(G_HERO_00_H-6)/4*40) == 12 ||
						   *(tilePared+(G_HERO_00_H-2)/4*40) == 12 ||
						   *(tilePared+(G_HERO_00_H)/4*40) == 12 )
		return 1;
	return 0;
}

/****************************************MOVIMIENTO****************************************/

//Limite derecho del mapa
#define ANCHO 80
#define LIMIT ANCHO - G_HERO_00_W

//Mover a la izquierda
void player_moveLeft() {
	if(!player_onWallLeft()){
		player.x--;
		player.moving = 2;
		player.facing = LEFT;
	}
}

//Mover a la derecha
void player_moveRight() {
	if(!player_onWallRight()){
		player.x++;
		player.moving = 2;
		player.facing = RIGHT;
	}
}

//Saltar
void player_jump() {
	player.vy = -6;
}

void player_fall(){
	if(player_onCeiling())
		player.vy=1;
	if(player.y + player.vy < ORIGEN_MAPA_Y)
		avanzarMapa();
	player.y += player.vy;
	if(player.vy < 3)
		player.vy += GRAVITY;
}

/****************************************DISPARO****************************************/


void player_shoot(){
	if(player.pShoot==NO && player.x > 8){
		projectile_initialize(0);
		player.pShoot=YES;
	}
}